def EuclideanDis(pts1, pts2):
    """
    欧式距离
    :param pts1: 位置（x1,y1）
    :param pts2: 位置（x2,y2）
    :return: 两点间距离
    """
    return ((pts2[0] - pts1[0]) ** 2 + (pts2[1] - pts1[1]) ** 2) ** 0.5









class pysort():
    def insertion_sort(self, arr):
        """
        直接插入排序
        :param arr:
        :return:
        """
        n = len(arr)
        for i in range(1, n):
            key = arr[i]
            j = i - 1
            while j >= 0 and arr[j] > key:
                arr[j + 1] = arr[j]
                j -= 1
            arr[j + 1] = key
            print(arr)

    def quick_sort(self, arr, low, high):
        """
        快速排序
        :param arr:
        :param low:
        :param high:
        :return:
        """
        if low < high:
            pivot_index = self.partition(arr, low, high)
            print(arr)
            self.quick_sort(arr, low, pivot_index - 1)
            self.quick_sort(arr, pivot_index + 1, high)

    def partition(self, arr, low, high):
        """
        :param arr:
        :param low:
        :param high:
        :return:
        """
        pivot = arr[high]
        i = low - 1
        for j in range(low, high):
            if arr[j] <= pivot:
                i += 1
                arr[i], arr[j] = arr[j], arr[i]
        arr[i + 1], arr[high] = arr[high], arr[i + 1]
        return i + 1

    def bubble_sort(self, arr):
        """
        冒泡排序
        :param arr:
        :return:
        """
        n = len(arr)
        for i in range(n - 1):
            for j in range(n - 1 - i):
                if arr[j] > arr[j + 1]:
                    arr[j], arr[j + 1] = arr[j + 1], arr[j]
            print(arr)

    def selection_sort(self, arr):
        """
        直接选择排序
        :param arr:
        :return:
        """
        n = len(arr)
        for i in range(n - 1):
            min_index = i
            for j in range(i + 1, n):
                if arr[j] < arr[min_index]:
                    min_index = j
            arr[i], arr[min_index] = arr[min_index], arr[i]
            print(arr)
if __name__=="__main__":
    a=[64,45,8,10,70,6]
    Sort=pysort()
    Sort.selection_sort(a)


import os
import shutil
import tempfile
from datetime import timedelta
from minio import Minio
from minio.error import S3Error
from minio.error import InvalidResponseError


def Creatclient(endpoint,access_key,secret_key,mode=1):
    """
    :param endpoint:  S3服务的主机名
    :param access_key: 用户ID
    :param secret_key: 密码
    :param mode: 默认为1，自定义，为0，表示使用游乐场测试与开发
    :return: 返回创建的客户端
    """
    if mode==0:
        client = Minio(
                endpoint="play.min.io",
                access_key="Q3AM3UQ867SPQQA43P2F",
                secret_key="zuf+tfteSlswRu7BJ86wekitnifILbZam1KYY3TG",
            )
    else:
        client = Minio(
            endpoint=endpoint,
            access_key=access_key,
            secret_key=secret_key,
        )
    return client

class Bucket():
    """操作桶"""
    def Foundbucket(self,client,bucket_name):
        """
        :param client: 客户端
        :param bucket_name: 存储桶名
        :return: 存在就继续进行，没有存在就创建一个桶
        """
        found = client.bucket_exists(bucket_name)
        if not found:
            client.make_bucket(bucket_name)
        else:
            print(f"Bucket {bucket_name} already exists")

    def get_bucket_list(self,client):
        """
        列出所有的存储桶名
        """
        try:
           buckets = client.list_buckets()
           for bucket in buckets:
               print(bucket.name, bucket.creation_date)  # 获取桶的名称和创建时间
        except InvalidResponseError as err:
            print(err)

    def get_remove_bucket(self,client,bucket_name):
        """
        删除存储桶
        """
        try:
            client.remove_bucket(bucket_name)
            print("删除存储桶成功")
        except InvalidResponseError as err:
            print(err)

    def get_bucket_files(self,client,bucket_name):
        """
        用于查看存储桶的对象
        """
        try:
            objects = client.list_objects(bucket_name, prefix=None,
                                               recursive=True)
            for obj in objects:
                print(obj.bucket_name, obj.object_name.encode('utf-8'), obj.last_modified,
                      obj.etag, obj.size, obj.content_type)
        except InvalidResponseError as err:
            print(err)

class Object():
    """操作对象
    example:
    MinIO3=MinIO.Object()
    MinIO3.upload_object(client, "ikun", r'D:\Python_zjr\python_minio\doc\ipcs.docx')
    MinIO3.upload_folder(client, "ikun","D:\Python_zjr\python_minio\doc")
    MinIO3.delete_object(client, "ikun",["main2.py","MinIO.py"] )
    MinIO3.delete_folder(client,"ikun","./doc")
    MinIO3.download_folder(client, "ikun", r"folder")
    MinIO.download_objects(client, "ikun",["main2.py","MinIO.py"] )
    """
    def delete_object(self,client, bucket_name, objects):
        """
        删除对象
        :param client:
        :param bucket_name:
        :param objects: 列表，元素为字符，逗号为间隔，如['myobject-1', 'myobject-2', 'myobject-3']
        :return:
        """
        try:
            for obj in objects:
                client.remove_object(bucket_name, obj)
                print(f"Deleted {obj} under {bucket_name}")
        except InvalidResponseError as err:
            print(err)

    def download_object(self,client, bucket_name, objects, filepath=None):
        """
        下载对象
        :param client:
        :param bucket_name:
        :param objects: 列表，元素为字符，逗号为间隔，如["main2.py","MinIO.py"]
        :param filepath: 默认为None，即是运行该文件的根目录下
        :return:
        """
        try:
            if filepath is None:
                current_dir = os.path.dirname(os.path.abspath(__file__))  # 获取当前文件的目录
            else:
                current_dir = filepath
            for obj in objects:
                file_path = os.path.join(current_dir, obj)  # 拼接目录和文件名
                client.fget_object(bucket_name, obj, file_path)
                print(f"Downloaded object {obj} to {file_path}")
        except InvalidResponseError as err:
            print(err)

    def upload_object(self,client, bucket_name, file_path):
        """
        上传对象
        :param client:
        :param bucket_name:
        :param file_path: 建议是绝对路径
        :return:
        """
        try:
            object_name = os.path.basename(file_path)  # 提取文件名作为对象名称
            with open(file_path, "rb") as file_data:
                file_size = os.path.getsize(file_path)
                client.put_object(bucket_name, object_name, file_data, file_size)
                print(f"Uploaded object {object_name} to bucket {bucket_name}")
        except IOError as e:
            print(f"Failed to open file: {file_path} - {e}")
        except S3Error as err:
            print(f"Error occurred: {err}")

    def delete_folder(self,client, bucket_name, folder_path):
        """
        删除的也是压缩包
        :param client:
        :param bucket_name:
        :param folder_path:
        :return:
        """
        try:
            temp_dir = tempfile.mkdtemp()
            folder_name = os.path.basename(folder_path)
            zip_file = os.path.join(temp_dir, f"{folder_name}.zip")

            shutil.make_archive(zip_file[:-4], 'zip', folder_path)

            client.remove_object(bucket_name, f"{folder_name}.zip")
            print(f"Deleted {folder_name}.zip from bucket {bucket_name}")

            shutil.rmtree(temp_dir)
        except InvalidResponseError as err:
            print(err)

    def upload_folder(self,client, bucket_name, folder_path):
        """
        上传的文件是zip压缩包
        :param client:
        :param bucket_name:
        :param folder_path:
        :return:
        """
        try:
            temp_dir = tempfile.mkdtemp()
            folder_name = os.path.basename(folder_path)
            zip_file = os.path.join(temp_dir, f"{folder_name}.zip")

            shutil.make_archive(zip_file[:-4], 'zip', folder_path)

            with open(zip_file, "rb") as file_data:
                file_size = os.path.getsize(zip_file)
                client.put_object(bucket_name, f"{folder_name}.zip", file_data, file_size)
                print(f"Uploaded {folder_name}.zip to bucket {bucket_name}")

            shutil.rmtree(temp_dir)
        except IOError as e:
            print(f"Failed to compress folder: {folder_path} - {e}")
        except S3Error as err:
            print(f"Error occurred: {err}")

    def download_folder(self,client, bucket_name, local_path):
        """
        下载后会自动解压
        :param client:
        :param bucket_name:
        :param local_path:
        :return:
        """
        try:
            temp_dir = tempfile.mkdtemp()

            client.fget_object(bucket_name, f"{local_path}.zip", os.path.join(temp_dir, f"{local_path}.zip"))
            print(f"Downloaded {local_path}.zip from bucket {bucket_name}")

            shutil.unpack_archive(os.path.join(temp_dir, f"{local_path}.zip"), local_path)
            print(f"Extracted {local_path}.zip to {local_path}")

            shutil.rmtree(temp_dir)
        except InvalidResponseError as err:
            print(err)

class Presigned():
    def upload_url(self, client, bucket_name, object_name, expires_in=7200):
        """
        生成预签名的上传 URL
        :param client: MinIO 客户端
        :param bucket_name: 存储桶名称
        :param object_name: 对象名称
        :param expires_in: URL 的有效期（单位：秒），默认为 2 小时
        :return: 预签名的上传 URL
        """
        try:
            # 生成预签名 URL
            url = client.presigned_put_object(bucket_name, object_name, expires=timedelta(seconds=expires_in))
            return url
        except Exception as e:
            print(f"Failed to generate presigned upload URL: {e}")
            return None

    def download_url(self,client, bucket_name, object_name, expires_in=7200):
        """
        生成预签名的下载 URL
        :param client: MinIO 客户端
        :param bucket_name: 存储桶名称
        :param object_name: 对象名称
        :param expires_in: URL 的有效期（单位：秒），默认为 2 小时
        :return: 预签名的下载 URL
        """
        try:
            # 生成预签名 URL
            url = client.presigned_get_object(bucket_name, object_name, expires=timedelta(seconds=expires_in))
            return url
        except Exception as e:
            print(f"Failed to generate presigned download URL: {e}")
            return None


import cv2
import math
import numpy as np
import os, shutil
from matplotlib import pyplot as plt

def stackImages(scale,imgArray):
    """
    :param scale:图片的规模,可使用小数,1为原始图像
    :param imgArray: 图像组成的列表，来表示行列排列
    :return: 生成的图像可按输入列表的行列排序展示,拼接展示图像
    """
    rows = len(imgArray)
    cols = len(imgArray[0])
    rowsAvailable = isinstance(imgArray[0], list)
    width = imgArray[0][0].shape[1]
    height = imgArray[0][0].shape[0]
    if rowsAvailable:
        for x in range ( 0, rows):
            for y in range(0, cols):
                if imgArray[x][y].shape[:2] == imgArray[0][0].shape [:2]:
                    imgArray[x][y] = cv2.resize(imgArray[x][y], (0, 0), None, scale, scale)
                else:
                    imgArray[x][y] = cv2.resize(imgArray[x][y], (imgArray[0][0].shape[1], imgArray[0][0].shape[0]), None, scale, scale)
                if len(imgArray[x][y].shape) == 2: imgArray[x][y]= cv2.cvtColor( imgArray[x][y], cv2.COLOR_GRAY2BGR)
        imageBlank = np.zeros((height, width, 3), np.uint8)
        hor = [imageBlank]*rows
        for x in range(0, rows):
            hor[x] = np.hstack(imgArray[x])
        ver = np.vstack(hor)
    else:
        for x in range(0, rows):
            if imgArray[x].shape[:2] == imgArray[0].shape[:2]:
                imgArray[x] = cv2.resize(imgArray[x], (0, 0), None, scale, scale)
            else:
                imgArray[x] = cv2.resize(imgArray[x], (imgArray[0].shape[1], imgArray[0].shape[0]), None,scale, scale)
            if len(imgArray[x].shape) == 2: 
                imgArray[x] = cv2.cvtColor(imgArray[x], cv2.COLOR_GRAY2BGR)
        hor= np.hstack(imgArray)
        ver = hor
    return ver

def addnoisy(image, n=10000):
    """
    :param image: 原始图像
    :param n: 添加椒盐的次数，默认为10000
    :return: 返回被椒盐处理后的图像
    """
    result = image.copy()
    w, h = image.shape[:2]
    for i in range(n):
        # 分别在宽和高的范围内生成一个随机值，用以代表x, y坐标
        x = np.random.randint(0, w)
        y = np.random.randint(0, h)
        if np.random.randint(0, 2) == 0:   #产生0，1的随机数
            # 生成白色噪声（盐噪声）
            result[x, y] = 0
        else:
            # 生成黑色噪声（椒噪声）
            result[x, y] = 255
    return result

def getContours(img, cThr=(100, 100), showCanny=False, minArea=1000, filter=0, draw=True):
    """
    :param img: 输入图像
    :param cThr: 阈值
    :param showCanny:展示经过处理后的边缘，默认Fales
    :param minArea: 更改大小
    :param filter: 过滤
    :param draw: 绘制边缘
    :return: 返回图像轮廓
    """
    imgGray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    imgBlur = cv2.GaussianBlur(imgGray, (5, 5), 1)
    imgCanny = cv2.Canny(imgBlur, cThr[0], cThr[1])
    kernel = np.ones((5, 5))
    imgDial = cv2.dilate(imgCanny, kernel, iterations=3)
    imgThre = cv2.erode(imgDial, kernel, iterations=2)
    if showCanny: cv2.imshow('Canny', imgThre)
    contours, hiearchy = cv2.findContours(imgThre, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
    finalCountours = []
    for i in contours:
        area = cv2.contourArea(i)
        if area > minArea:
            peri = cv2.arcLength(i, True)
            approx = cv2.approxPolyDP(i, 0.02 * peri, True)
            bbox = cv2.boundingRect(approx)
            if filter > 0:
                if len(approx) == filter:
                    finalCountours.append([len(approx), area, approx, bbox, i])
            else:
                finalCountours.append([len(approx), area, approx, bbox, i])
    finalCountours = sorted(finalCountours, key=lambda x: x[1], reverse=True)
    if draw:
        for con in finalCountours:
            cv2.drawContours(img, con[4], -1, (0, 0, 255), 3)
    return img, finalCountours


def empty(a):
    pass




import cv2
import numpy as np
from pyzjr.ColorModule import ColorFinder
import pyzjr.utils as zjr
import logging

def HSV(mode):
    """
    * 示例仅供参考，只是为了在调试过程中能够更快的找到相要的颜色
     可以进行调试成功的值进行替换，但也要记住将原来颜色的值注释。
    :param mode:
    :return:
    """
    if mode == 'black':
        Min_and_MaxVal = [[0, 0, 0], [180, 255, 46]]
        return Min_and_MaxVal
    elif mode == 'gray':
        Min_and_MaxVal = [[0, 0, 46], [180, 43, 220]]
        return Min_and_MaxVal
    elif mode == 'white':
        Min_and_MaxVal = [[0, 0, 221], [180, 30, 225]]
        return Min_and_MaxVal
    elif mode == 'red':
        Min_and_MaxVal = [[156, 43, 46], [180, 255, 225]]
        # Min_and_MaxVal = [156, 43, 46], [180, 255, 225] or [0, 43, 46], [10, 255, 225]
        return Min_and_MaxVal
    elif mode == 'orange':
        Min_and_MaxVal = [[11, 43, 46], [25, 255, 225]]
        return Min_and_MaxVal
    elif mode == 'yellow':
        Min_and_MaxVal = [[26, 43, 46], [34, 255, 225]]
        return Min_and_MaxVal
    elif mode == 'green':
        Min_and_MaxVal = [[28,38,36],[64,255,255]]
        # 参考值：[[35, 43, 46], [77, 255, 225]]
        return Min_and_MaxVal
    elif mode == 'cyan':
        Min_and_MaxVal = [[78, 43, 46], [99, 255, 225]]
        return Min_and_MaxVal
    elif mode == 'blue':
        Min_and_MaxVal = [[100, 43, 46], [124, 255, 225]]
        return Min_and_MaxVal
    elif mode == 'purple':
        Min_and_MaxVal = [[125, 43, 46], [155, 255, 225]]
        return Min_and_MaxVal
    elif mode == 0:
        print("请按照‘,’分隔分别输入hmin, smin, vmin, hmax, smax, vmax的值", end='/n')
        return 0
    else:
        logging.warning("颜色并不在示例范围内")




class getMask():
    def __init__(self, trackBar=True):
        self.ColF = ColorFinder(trackBar)
        # self.ColF.initTrackbars()
    def protect_region(self, mask ,threshold=None):
        """
        * 用于保护掩膜图的部分区域
        :param mask: 掩膜图
        :param threshold: 如果为None,则为不保护，如果是长为4的列表，则进行特定区域的保护
        :return: 返回进行保护区域的掩膜图

        example:    [0, img.shape[1], 0, img.shape[0]]为全保护状态，
                    x_start可以保护大于x的部分
                    x_end可以保护小于x的部分
                    y_start可以保护图像下方的部分
                    y_end可以保护图像上方的部分
        """
        if threshold == None:
            return mask
        else:
            x_start, x_end, y_start, y_end = threshold[:4]
            mask[y_start:y_end, x_start:x_end] = 0
            return mask

    def MaskZone(self, img, HsvVals):
        """
        * 生成掩膜图以及融合图像
        :param img: 输入图像
        :param HsvVals: 可以通过getTrackbarValues获得
        :return: 返回融合图、掩膜图、HSV图
        """
        imgHSV = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
        lower = np.array(HsvVals[0])
        upper = np.array(HsvVals[1])
        mask = cv2.inRange(imgHSV, lower, upper)
        imgResult = cv2.bitwise_and(img, img, mask=mask)
        return imgResult, mask, imgHSV

    def DetectImg(self, path, ConsoleOut=True, threshold=None, scale=1.0):
        """
        * 轨迹栏检测图片
        :param path: 图片路径
        :param ConsoleOut: 用于是否控制台打印HsvVals的值
        :param threshold: 阈值，用于保护图片的区域
        :param scale: 图片规模大小
        :return: 无返回
        """
        while True:
            img = cv2.imread(path)
            HsvVals = self.ColF.getTrackbarValues(False)
            if ConsoleOut:
                print(HsvVals)
            imgResult, mask, imgHSV = self.MaskZone(img,HsvVals)
            pro_mask = self.protect_region(mask, threshold)
            imgStack = zjr.stackImages(scale, ([img,imgHSV],[pro_mask,imgResult]))
            cv2.imshow("Stacked Images", imgStack)
            k = cv2.waitKey(1)
            if k == 27:
                break


if __name__=="__main__":
    path = r'E:\pythonProject2\pyzjr\resources\AI.png'
    img2 = cv2.imread(path)
    getMask=getMask()
    getMask.DetectImg(path,threshold=[0, img2.shape[1], 300, img2.shape[0]],scale=0.4)








from tqdm import tqdm
import requests
import cv2
import os, shutil
from matplotlib import pyplot as plt
from PIL import Image


def download_file(url):
    """
    :param url:下载文件所在url链接
    :return: 下载的位置处于根目录
    """
    print("------", "Start download with urllib")
    name = url.split("/")[-1]
    resp = requests.get(url, stream=True)
    content_size = int(resp.headers['Content-Length']) / 1024  # 确定整个安装包的大小
    # 下载到上一级目录
    path = os.path.abspath(os.path.dirname(os.getcwd())) + "\\" + name
    # 下载到该目录
    path = os.getcwd() + "\\" + name
    print("File path:  ", path)
    with open(path, "wb") as file:
        print("File total size is:  ", content_size)
        for data in tqdm(iterable=resp.iter_content(1024), total=content_size, unit='k', desc=name):
            file.write(data)
    print("------", "finish download with urllib\n\n")

def getPhotopath(paths):
    """
    * 批量读取文件夹下的图片路径
    :param paths: 文件夹路径
    :return: 包含图片路径的列表
    """
    imgfile = []
    file_list = os.listdir(paths)
    for i in file_list:
        newph = os.path.join(paths, i)
        imgfile.append(newph)
    return imgfile



def Pic_rename(img_folder,object='Crack',format='jpg',num=0):
    """
    * 用于批量修改图像的命名
    :param img_folder:存放图片的路径
    :param object: 图像的对象
    :param format: 图片格式,可自行命名,这里给出jpg
    :param num: 对图片进行计数
    :return: 用dst替换src
    """
    for img_name in os.listdir(img_folder):
        src = os.path.join(img_folder, img_name)
        dst = os.path.join(img_folder, object+ str(num) +'.'+ format)
        num= num+1
        os.rename(src, dst)

def read_resize_image(path, size=1.0, show=False, space=False):
    """
    :param path:图片的路径
    :param size: 希望得到图片的大小,cat.0为原始图像
    :param show: 如果为真，返回原始图像和修改后图像的长、宽、通道
    :param space: 如果为真，返回图像的灰度图
    :return: 返回修改大小后的图像
    """
    if space==False:
        originalimage = cv2.imread(path)
    else:
        originalimage = cv2.imread(path,0)
    if size != 1.0:
        height, width = originalimage.shape[:2]
        size = (int(width * size), int(height * size))
        originalimage = cv2.resize(originalimage, size, interpolation=cv2.INTER_AREA)
    if not show:
        pass
    else:
        print(originalimage)
        print(originalimage.shape)
    return originalimage


def load_images_from_folder(folder_path):
    """
    加载一个文件夹下的图片，并存入列表中并返回
    :param folder_path: 
    :return: 
    """
    images = []
    for filename in os.listdir(folder_path):
        if filename.endswith(".jpg") or filename.endswith(".png"):
            img_path = os.path.join(folder_path, filename)
            img = cv2.imread(img_path)
            if img is not None:
                images.append(img)
    return images

def save_images(images, folder_path):
    """
    批量保存一图片
    :param images: load_images_from_folder的返回值
    :param folder_path: 保存路径
    :return: 无返回
    """
    os.makedirs(folder_path, exist_ok=True)
    for i, img in enumerate(images):
        img_path = os.path.join(folder_path, f"image_{i}.jpg")
        cv2.imwrite(img_path, img)



"""
- author:Auorui(夏天是冰红茶)
- creation time:2022.11.19
- blog:https://blog.csdn.net/m0_62919535/article/details/127818006
"""
import cv2
import math
from pyzjr.FM import getPhotopath
import numpy as np
import pyzjr.Z as Z

class Clear_quantification():
    def brenner(self,img):
        '''
        Brenner梯度函数
        :param img:narray 二维灰度图像
        :return: int 图像越清晰越大
        '''
        shapes = np.shape(img)
        output = 0
        for x in range(0, shapes[0] - 2):
            for y in range(0, shapes[1]):
                output += (int(img[x + 2, y]) - int(img[x, y])) ** 2
        return output

    def EOG(self,img):
        '''
        能量梯度函数(Energy of Gradient)
        :param img:narray 二维灰度图像
        :return: int 图像越清晰越大
        '''
        shapes = np.shape(img)
        output = 0
        for x in range(0, shapes[0] - 1):
            for y in range(0, shapes[1] - 1):
                output += ((int(img[x + 1, y]) - int(img[x, y])) ** 2 + (int(img[x, y + 1]) - int(img[x, y])) ** 2)
        return output

    def Roberts(self,img):
        '''
        Roberts函数
        :param img:narray 二维灰度图像
        :return: int 图像越清晰越大
        '''
        shapes = np.shape(img)
        output = 0
        for x in range(0, shapes[0] - 1):
            for y in range(0, shapes[1] - 1):
                output += ((int(img[x + 1, y + 1]) - int(img[x, y])) ** 2 + (
                            int(img[x + 1, y]) - int(img[x, y + 1])) ** 2)
        return output

    def Laplacian(self,img):
        '''
        Laplace
        :param img:narray 二维灰度图像
        :return: int 图像越清晰越大
        '''
        return cv2.Laplacian(img, Z.Lap_64F).var()

    def SMD(self,img):
        '''
        SMD（灰度方差）函数
        :param img:narray 二维灰度图像
        :return: int 图像越清晰越大
        '''
        shape = np.shape(img)
        output = 0
        for x in range(1, shape[0] - 1):
            for y in range(0, shape[1]):
                output += math.fabs(int(img[x, y]) - int(img[x, y - 1]))
                output += math.fabs(int(img[x, y] - int(img[x + 1, y])))
        return output

    def SMD2(self,img):
        '''
        （灰度方差乘积）函数
        :param img:narray 二维灰度图像
        :return: int 图像约清晰越大
        '''
        shape = np.shape(img)
        output = 0
        for x in range(0, shape[0] - 1):
            for y in range(0, shape[1] - 1):
                output += math.fabs(int(img[x, y]) - int(img[x + 1, y])) * math.fabs(
                    int(img[x, y] - int(img[x, y + 1])))
        return output

class Fuzzy_image():
    def getImgVar(self,image):
        """
        :param image: 图像
        :return: 图片清晰度数值
        """
        quanti=Clear_quantification()
        imggray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        imageVar = quanti.Laplacian(imggray)
        return imageVar

    def getTest(self,imgfile):
        """
        :param imgfile: getPhotopath()
        :return: 模糊值排序
        """
        c = []
        for i in imgfile:
            # print(i)
            img = cv2.imread(i)
            image = self.getImgVar(img)
            # print(image)
            c.append(float(f"{image:.3f}"))
        if 'test' in imgfile[0]:  # 对测试集数据进行反转
            c.sort(reverse=True)
        else:
            c.sort()
        return c

    def getThr(self,pathTest = "./ces/test",pathStd = "./ces/Standards"):
        """
        :param pathTest:测试的数据集文件夹位置
        :param pathStd: 标准图的数据文件夹位置
        :return: 模糊判定未知区间
        """
        imgfile1 = getPhotopath(pathTest)
        imgfile2 = getPhotopath(pathStd)
        a = self.getTest(imgfile1)
        b = self.getTest(imgfile2)
        thr = (a[0], b[0])
        # print(thr)
        return thr

def vagueJudge(image, show_minandmax=True):
    """
    :param image: 图片
    :param show_minandmax:是否输出模糊未知区间
    :return: 模糊数值打印在图片上显示
    """
    Fuzzy = Fuzzy_image()
    img = cv2.imread(image)
    imgVar = Fuzzy.getImgVar(img)
    minThr, maxThr = Fuzzy.getThr()
    if imgVar > maxThr:
        cv2.putText(img, f"Not Vague{imgVar:.2f}", (12, 70), cv2.FONT_HERSHEY_PLAIN, 3,
                        (255, 0, 0), 3)
    else:
        cv2.putText(img, f"Vague{imgVar:.2f}", (12, 70), cv2.FONT_HERSHEY_PLAIN, 3,
                        (255, 0, 0), 3)
    if show_minandmax:
        print(minThr, maxThr)
    else:
        pass
    cv2.imshow("img", img)
    k = cv2.waitKey(0) & 0xFF


if __name__ == "__main__":
    image="./test/01.jpg"   #需要进行测试的图片
    vagueJudge(image)
